import os

from articles.models import Article
from django.conf import settings
from django.urls import reverse

from django_ckeditor_5.signals import extract_image_paths
from django_ckeditor_5.storage_utils import get_storage_class


def test_cleanup_ckeditor_images_on_delete(file, admin_client):
    with file as upload:
        upload_view_name = getattr(
            settings,
            "CK_EDITOR_5_UPLOAD_FILE_VIEW_NAME",
            "ck_editor_5_upload_file",
        )
        response = admin_client.post(
            reverse(upload_view_name),
            {"upload": upload},
        )
    assert response.status_code == 200
    assert "url" in response.json()
    article = Article.objects.create(
        title="test", text=f"<img src='{response.json()['url']}'>"
    )
    images = extract_image_paths(article.text)
    storage = get_storage_class()
    for image in images:
        f_url = os.path.join(storage.location, image)
        file_name = os.path.basename(f_url)
        abs_path = os.path.join(storage.location, file_name)
        assert os.path.exists(abs_path)
    article.delete()
    for image in images:
        f_url = os.path.join(storage.location, image)
        file_name = os.path.basename(f_url)
        abs_path = os.path.join(storage.location, file_name)
        assert not os.path.exists(abs_path)



from django import get_version
from django.views.decorators.http import require_POST

from .exceptions import NoImageException
from .permissions import check_upload_permission
from .storage_utils import image_verify, handle_uploaded_file

if get_version() >= "4.0":
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _

from django.conf import settings
from django.http import JsonResponse

from .forms import UploadFileForm


@require_POST
@check_upload_permission
def upload_file(request):
    form = UploadFileForm(request.POST, request.FILES)
    allow_all_file_types = getattr(settings, "CKEDITOR_5_ALLOW_ALL_FILE_TYPES", False)

    if not allow_all_file_types:
        try:
            image_verify(request.FILES["upload"])
        except NoImageException as ex:
            return JsonResponse({"error": {"message": f"{ex}"}}, status=400)

    if form.is_valid():
        url = handle_uploaded_file(request.FILES["upload"])
        return JsonResponse({"url": url})

    if form.errors["upload"]:
        return JsonResponse(
            {"error": {"message": form.errors["upload"][0]}},
            status=400,
        )

    return JsonResponse({"error": {"message": _("Invalid form data")}}, status=400)



from django.apps import AppConfig


class DjangoCkeditor5Config(AppConfig):
    name = "django_ckeditor_5"
    verbose_name = "Django CKEditor 5"

    def ready(self):
        from . import signals  # noqa: F401



from django.conf import settings

from django_ckeditor_5.views import upload_file


def test_upload_file_permission_anonymous(factory, anonymous_user, file):
    settings.CKEDITOR_5_FILE_UPLOAD_PERMISSION = "authenticated"
    request = factory.post("/upload/", {"upload": file})
    request.user = anonymous_user
    response = upload_file(request)
    assert response.status_code == 403


def test_upload_file_permission_authenticated(factory, authenticated_user, file):
    settings.CKEDITOR_5_FILE_UPLOAD_PERMISSION = "authenticated"
    request = factory.post("/upload/", {"upload": file})
    request.user = authenticated_user
    response = upload_file(request)
    assert response.status_code == 200


def test_upload_file_permission_staff(factory, staff_user, file):
    settings.CKEDITOR_5_FILE_UPLOAD_PERMISSION = "staff"
    request = factory.post("/upload/", {"upload": file})
    request.user = staff_user
    response = upload_file(request)
    assert response.status_code == 200


def test_upload_file_permission_any(factory, anonymous_user, file):
    settings.CKEDITOR_5_FILE_UPLOAD_PERMISSION = "any"
    request = factory.post("/upload/", {"upload": file})
    request.user = anonymous_user
    response = upload_file(request)
    assert response.status_code == 200


def test_upload_file_permission_authenticated_user(factory, authenticated_user, file):
    settings.CKEDITOR_5_FILE_UPLOAD_PERMISSION = "any"
    request = factory.post("/upload/", {"upload": file})
    request.user = authenticated_user
    response = upload_file(request)
    assert response.status_code == 200



"""blog URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.conf import settings
from django.conf.urls.static import serve
from django.contrib import admin
from django.urls import include, path, re_path

urlpatterns = [
    path("admin/", admin.site.urls),
    path("", include("articles.urls")),
    path("ckeditor5/", include("django_ckeditor_5.urls")),
    re_path(r"^media/(?P<path>.*)$", serve, {"document_root": settings.MEDIA_ROOT}),
    re_path(r"^static/(?P<path>.*)$", serve, {"document_root": settings.STATIC_ROOT}),
]



class NoImageException(Exception):
    """
    Exception raised when the file is not a valid image.
    """

    def __init__(self, message="File is not a valid image"):
        self.message = message
        super().__init__(self.message)



from django import forms
from django.conf import settings
from django.core.validators import FileExtensionValidator

from django_ckeditor_5.validators import FileMaxSizeValidator


class UploadFileForm(forms.Form):
    upload = forms.FileField(
        validators=[
            FileExtensionValidator(
                getattr(
                    settings,
                    "CKEDITOR_5_UPLOAD_FILE_TYPES",
                    ["jpg", "jpeg", "png", "gif", "bmp", "webp", "tiff"],
                ),
            ),
            FileMaxSizeValidator(getattr(settings, "CKEDITOR_5_MAX_FILE_SIZE", 0)),
        ],
    )



from django_ckeditor_5.widgets import CKEditor5Widget


def test_ckeditor5_field_internal_type(ckeditor5_field):
    assert ckeditor5_field.get_internal_type() == "TextField"


def test_ckeditor5_field_formfield(ckeditor5_field):
    formfield = ckeditor5_field.formfield()
    assert formfield.max_length == ckeditor5_field.max_length
    assert isinstance(formfield.widget, CKEditor5Widget)



import os

import pytest
from django.contrib.auth.models import AnonymousUser, User
from django.test import RequestFactory

from django_ckeditor_5.fields import CKEditor5Field
from django_ckeditor_5.forms import UploadFileForm


@pytest.fixture
def file():
    file_path = os.path.join(os.path.dirname(__file__), "fixtures", "files", "test.png")
    return open(file_path, "rb")


@pytest.fixture
def ckeditor5_field():
    return CKEditor5Field()


@pytest.fixture
def upload_file_form():
    return UploadFileForm()


@pytest.fixture
def factory():
    return RequestFactory()


@pytest.fixture
def anonymous_user():
    return AnonymousUser()


@pytest.fixture
def authenticated_user(db):  # noqa: ARG001
    return User.objects.create_user(username="testuser", password="12345")  # noqa: S106


@pytest.fixture
def staff_user(db):  # noqa: ARG001
    return User.objects.create_user(
        username="staffuser",
        password="12345",  # noqa: S106
        is_staff=True,
    )



import os

from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.urls import reverse


def test_upload_file(admin_client, file):
    with file as upload:
        upload_view_name = getattr(
            settings,
            "CK_EDITOR_5_UPLOAD_FILE_VIEW_NAME",
            "ck_editor_5_upload_file",
        )
        response = admin_client.post(
            reverse(upload_view_name),
            {"upload": upload},
        )
    assert response.status_code == 200
    assert "url" in response.json()


def test_upload_file_too_big(admin_client):
    file = SimpleUploadedFile("test_image_big.jpeg", bytearray(os.urandom(62915)))
    upload_view_name = getattr(
        settings,
        "CK_EDITOR_5_UPLOAD_FILE_VIEW_NAME",
        "ck_editor_5_upload_file",
    )
    response = admin_client.post(
        reverse(upload_view_name),
        {"upload": file},
    )
    assert response.status_code == 400
    response_data = response.json()
    assert "error" in response_data
    error = response_data["error"]
    assert "message" in error
    assert error["message"] == "File should be at most 0.06 MB."


def test_upload_file_forbbiden_file_typ(admin_client):
    file = SimpleUploadedFile("test_pdf.pdf", b"random data")
    upload_view_name = getattr(
        settings,
        "CK_EDITOR_5_UPLOAD_FILE_VIEW_NAME",
        "ck_editor_5_upload_file",
    )
    response = admin_client.post(
        reverse(upload_view_name),
        {"upload": file},
    )
    assert response.status_code == 400
    response_data = response.json()
    assert "error" in response_data
    error = response_data["error"]
    assert "message" in error
    assert error["message"] == (
        "File extension “pdf” is not allowed. Allowed extensions are: jpg, jpeg, png, gif, "
        "bmp, webp, tiff."
    )



from django import get_version
from django.conf import settings
from django.http import JsonResponse

if get_version() >= "4.0":
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _


def check_upload_permission(view_func):
    def _wrapped_view(request, *args, **kwargs):
        permission = getattr(settings, "CKEDITOR_5_FILE_UPLOAD_PERMISSION", "staff")
        if permission == "staff" and not request.user.is_staff:
            return JsonResponse(
                {
                    "error": {
                        "message": _("You do not have permission to upload files."),
                    },
                },
                status=403,
            )
        if permission == "authenticated" and not request.user.is_authenticated:
            return JsonResponse(
                {"error": {"message": _("You must be logged in to upload files.")}},
                status=403,
            )
        return view_func(request, *args, **kwargs)

    return _wrapped_view
